/***************************************************************************
 *   Copyright (C) 2004 by bysk                                           *
 *   buysk@pf.pl                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <qprocess.h>

#include "kadu.h"
#include "userbox.h"
#include "config_dialog.h"

#include "weather.h"

#define MODULE_WEATHER_VERSION 0.1

Weather* weather_session;

extern "C" int weather_init()
{
    weather_session = new Weather();

    return 0;
}

extern "C" void weather_close()
{	
    delete weather_session;
}

SearchIDLocation::SearchIDLocation(ConfigFile* cf, const QStringList& wCfFiles,
                                   QWidget* parent, QString wTitle)
:
	QDialog(parent, wTitle),
	parser(strPage, cf),
	weatherConfigFiles(wCfFiles)
{

	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
}

SearchIDLocation::~SearchIDLocation()
{
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
}

bool SearchIDLocation::findID(QString loc_name, int config_pos)
{
	locName = loc_name;
	cfPos = config_pos;
	
	strPage = "";	
	QString wcfName = weatherConfigFiles[config_pos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
		
	QString encoding = wConfig.readEntry("Default","Encoding");
	decoder = QTextCodec::codecForName(encoding);
	
	httpclient.setHost(wConfig.readEntry("Name Search","Search host"));
	
	QString url;
	if(loc_name.isEmpty())
	{
		if(wConfig.readBoolEntry("Name Search","AllLocations"))
			url = wConfig.readEntry("Name Search","AllLocationsPath");
		else
			return false;
	}
	else
	{
		encodeUrl(loc_name, encoding);
		url.sprintf(wConfig.readEntry("Name Search","Search path"),(const char*)loc_name);
	}
	
	httpclient.get(url);
	return true;
}

void SearchIDLocation::downloadingFinished()
{
	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
	
	names.clear();
	codes.clear();
	parser.getSearch(names, codes, &wConfig);
	
	for(uint i = 0; i < names.count(); i++)
	{
		// Jeli strona zawiera wszystkie miasta,
		// trzeba dokadniej przefiltrowa
		if(wConfig.readBoolEntry("Name Search","OnePage") &&
			names[i].find(locName, 0, false) == -1)
		{
			names.remove(names.at(i));
			codes.remove(codes.at(i));
			i--;
			continue;	
		}
	}
}

void SearchIDLocation::downloadingError()
{


}

void SearchIDLocation::downloadingRedirected(QString link)
{
	httpclient.setHost("");
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));

	names.clear();
	codes.clear();
	names.push_back(locName);
	codes.push_back(parser.getFastSearch(link, wConfig));
}

void SearchIDLocation::encodeUrl(QString& url, const QString& enc)
{
	QUrl::encode(url);
	
	if(enc == "ISO8859-2")
	{
		//QString rep[18][2]={{"","%B1"},{"","%E6"},{"","%EA"},{"","%B3"},{"%","%F1"},{"","%F3"},{"","%B6"},{"","%BF"},{"","%BC"},
		//					{"","%A1"},{"","%C6"},{"","%CA"},{"","%A3"},{"","%D1"},{"","%D3"},{"","%A6"},{"","%AF"},{"","%AC"}};
	
		QString rep[18][2]={{"%C4%85","%B1"},{"%C4%87","%E6"},{"%C4%99","%EA"},{"%C5%84","%F1"},{"%C5%82","%B3"},{"%C3%B3","%F3"},{"%C5%9B","%B6"},{"%C5%BC","%BF"},{"%C5%BA","%BC"},
							{"%C4%84","%A1"},{"%C4%86","%C6"},{"%C4%98","%CA"},{"%C5%83","%D1"},{"%C5%81","%A3"},{"%C3%93","%D3"},{"%C5%9A","%A6"},{"%C5%BB","%AF"},{"%C5%B9","%AC"}};
	
		for(int i=0; i<18; i++)
		{
			url.replace(rep[i][0],rep[i][1]);
		}
	}
}

/**
	WeatherCodeDialog
**/

int WeatherCodeDialog::config_pos = 0;

WeatherCodeDialog::WeatherCodeDialog(const QString& altNick, QListViewItem* item, ConfigFile* cf,
			const QStringList& wCfFiles, const QStringList& wCfNames, int actual, QWidget* parent)
:
	listItem(item),
	moduleConfig(cf),
	weatherConfigNames(wCfNames),
	SearchIDLocation(cf, wCfFiles, parent, "WeatherCodeDialog")
{
	if(actual != -1)
		config_pos = actual;

	setWFlags(WDestructiveClose/* | WShowModal*/);

	QGridLayout *grid= new QGridLayout(this, 1,0, 5, 10);
	
	QHBox* config_box = new QHBox(this);
	new QLabel(tr("Weather server: "), config_box);
	c_config = new QComboBox(false,config_box);
	c_config->insertStringList(weatherConfigNames);
	c_config->setCurrentItem(config_pos);
	
	QVGroupBox* search_group_box = new QVGroupBox(tr("Search location code"),this);
	
	QHBox* search_box = new QHBox(search_group_box);
	new QLabel(QString(tr("Name: ")), search_box);
	e_name = new QLineEdit(search_box);
	b_search = new QPushButton(search_box);
	b_search->setText(tr("Search"));
	b_search->setMaximumWidth(200);
	
	lv_search = new QListView(search_group_box);
	lv_search->setSorting(-1);
	lv_search->addColumn(tr("Location"));
	lv_search->addColumn(tr("Code"));
	lv_search->setAllColumnsShowFocus(true);
	lv_search->setColumnWidthMode(0, QListView::Maximum);
	lv_search->setColumnWidthMode(1, QListView::Maximum);
	lv_search->setResizeMode(QListView::LastColumn);
	lv_search->setSorting(0);
	
	p_linking = new QProgressBar(0, search_group_box);
	p_linking->hide();
	
	QHBox* bottom = new QHBox(this);
	new QLabel(QString(tr("Location code for %1: ")).arg(altNick),bottom);
	e_id = new QLineEdit(item->text(1), bottom);
	
	QHBox* okcancel=new QHBox(this);
	
	b_ok = new QPushButton(okcancel);
	b_ok->setText(tr("OK"));
	b_ok->setMaximumWidth(200);
	b_ok->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	
	b_cancel = new QPushButton(okcancel);
	b_cancel->setText(tr("Cancel"));
	b_cancel->setMaximumWidth(200);
	b_cancel->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	
	grid->addWidget(config_box, 0, 0);
	grid->addWidget(search_group_box, 1, 0);
	grid->addWidget(bottom, 2, 0);
	grid->addWidget(okcancel, 3, 0, Qt::AlignRight);
	
	timer = new QTimer(this);
	
	setCaption(tr("Edit location code"));
	e_name->setFocus();
	
	connect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	connect(b_ok, SIGNAL(clicked()), this, SLOT(okButton()));
	connect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	connect(lv_search, SIGNAL(selectionChanged()), this, SLOT(searchSelChanged()));
	connect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
}

WeatherCodeDialog::~WeatherCodeDialog()
{	
	disconnect(b_search, SIGNAL(clicked()), this, SLOT(searchButton()));
	disconnect(b_ok, SIGNAL(clicked()), this, SLOT(okButton()));
	disconnect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	disconnect(lv_search, SIGNAL(selectionChanged()), this, SLOT(searchSelChanged()));
	disconnect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
}

void WeatherCodeDialog::okButton()
{
	QString code = e_id->text();
	listItem->setText(1, code);
	
	if(code.isEmpty())
	{
		listItem->setText(2, "");
		listItem->setText(3, "");
	}
	else
	{
		listItem->setText(2, weatherConfigNames[c_config->currentItem()]);
		listItem->setText(3, weatherConfigFiles[c_config->currentItem()]);
	}
	close();
}

void WeatherCodeDialog::searchButton()
{
	lv_search->clear();
	
	config_pos = c_config->currentItem();
	QString loc_name = e_name->text();
	if(!findID(loc_name, config_pos))
		QMessageBox::information(this,"",tr("List of locations not available for this server. Type part of location name."));
	else
	{
		p_linking->reset();
		p_linking->show();
		timer->start(100,false);
	}
	
}

void WeatherCodeDialog::downloadingFinished()
{
	timer->stop();
	p_linking->hide();
	
	SearchIDLocation::downloadingFinished();

	int count = names.count();
	for(uint i = 0; i < count; i++)
	{
		new QListViewItem(lv_search,names[i], codes[i]);
	}
	
	if(count == 0)
		QMessageBox::information(this, "", tr("Search location not found"));
	else if(count == 1)
	{
		lv_search->setCurrentItem(lv_search->firstChild());
		lv_search->setSelected(lv_search->firstChild(), true);
	}
}

void WeatherCodeDialog::downloadingError()
{
	p_linking->hide();
	timer->stop();
	QMessageBox::critical(this, "", tr("Downloading failed"));
}

void WeatherCodeDialog::downloadingRedirected(QString link)
{
	p_linking->hide();
	timer->stop();
	
	SearchIDLocation::downloadingRedirected(link);
	
	if(!(*codes.begin()).isEmpty())
	{
		QListViewItem* item = new QListViewItem(lv_search,*names.begin(), *codes.begin());
		lv_search->setCurrentItem(item);
		lv_search->setSelected(item, true);
	}
}

void WeatherCodeDialog::downloadingProgress()
{
	p_linking->setProgress(p_linking->progress()+10);
}

void WeatherCodeDialog::searchSelChanged()
{
	e_id->setText(lv_search->currentItem()->text(1));
}

ShowForecastDialog::ShowForecastDialog(const QString& uin, ConfigFile* cf, QWidget* parent)
	: QDialog(parent, "ShowForecastDialog"),
	moduleConfig(cf),
	parser(strPage,cf)
{
	loc_id = moduleConfig->readEntry("Locations",uin);
	QString cffile = moduleConfig->readEntry("Configs",uin);
	QString wconf;
	if(cffile.isNull())
		wconf = moduleConfig->readEntry("Weather","ServerConfigFile");
	else
	{
		wconf = dataPath("kadu/modules/data/weather/"+cffile);
		iconpath = dataPath("kadu/modules/data/weather/icons/");
	}
	
	wConfig = new ConfigFile(wconf);
	
	setWFlags(WDestructiveClose);
	
	QGridLayout* grid= new QGridLayout(this, 1,0, 5, 10);
	e_message = new QTextEdit(this);
	e_message->setTextFormat(QTextEdit::RichText);
	e_message->setReadOnly(true);
	
	p_linking = new QProgressBar(0, this);
	
	grid->addWidget(e_message, 0, 0);
	grid->addWidget(p_linking, 1, 0);
	
	timer = new QTimer(this);
	
	resize(300,300);
	
	for(UserList::const_iterator u = userlist.begin(); u != userlist.end(); u++)
	{
		if(QString().setNum((*u).uin()) == uin)
		{
			setCaption((*u).altNick() + " - " + tr("Forecast"));
			break;
		}
	}
	if(caption().isEmpty())
		setCaption(tr("Forecast"));
	
	connect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	
	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	
	decoder = QTextCodec::codecForName(wConfig->readEntry("Default","Encoding"));
	
	httpclient.setHost(wConfig->readEntry("Default","Default host"));
	QString url;
	url.sprintf(wConfig->readEntry("Default","Default path"),(const char*)loc_id);
	
	httpclient.get(url);
	
	p_linking->reset();
	timer->start(100,false);
}

ShowForecastDialog::~ShowForecastDialog()
{
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	
	disconnect(timer, SIGNAL(timeout()), this, SLOT(downloadingProgress()));
	
	delete wConfig;
}

void ShowForecastDialog::downloadingFinished()
{	
	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	p_linking->hide();
	timer->stop();
	
	QString Data = parser.getData(wConfig, iconpath);
	e_message->setText(Data);
}

void ShowForecastDialog::downloadingError()
{
	p_linking->hide();
	timer->stop();
	QMessageBox::critical(this, "", tr("Downloading failed"));
}

void ShowForecastDialog::downloadingProgress()
{
	p_linking->setProgress(p_linking->progress()+10);
}

/**
	Weather
**/
Weather::Weather()
{
	QString config_path = ggPath("weather.conf");
	printf("%s\n", config_path.ascii());
	moduleConfig=new ConfigFile(config_path);
	
	QDir dir(dataPath("kadu/modules/data/weather"),"*.ini");
	dir.setFilter(QDir::Files);
	for(unsigned int i=0; i<dir.count(); i++)
	{
		QString name=dir[i];
		weatherConfigFiles.append(name.left(name.length()));
		
		ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+name));
		weatherConfigNames.append(wConfig.readEntry("Header","Name"));
	}
	
	ConfigDialog::addTab(QT_TRANSLATE_NOOP("@default", "Weather"),"WeatherTab");
			
	ConfigDialog::addHBox("Weather", "Weather", "loc_box");
	ConfigDialog::addListView("Weather", "loc_box", "loc_list");
	ConfigDialog::addVBox("Weather", "loc_box", "loc_buttons_box");
	ConfigDialog::addPushButton("Weather", "loc_buttons_box", QT_TRANSLATE_NOOP("@default", "Edit"));

	ConfigDialog::registerSlotOnCreate(this, SLOT(CreateConfigDialog()));
	ConfigDialog::registerSlotOnApply(this, SLOT(ApplyConfigDialog()));
	ConfigDialog::connectSlot("Weather", "Edit", SIGNAL(clicked()), this, SLOT(EditLocation()));
	ConfigDialog::connectSlot("Weather", "loc_list",
			SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), this,
			SLOT(EditLocation()));
	
	UserBox::userboxmenu->addItemAtPos(2,"Show weather", tr("Show weather"), this, SLOT(ShowWeather()));
	menuMyWeather = kadu->mainMenu()->insertItem(tr("Show weather"), this, SLOT(ShowMyWeather()), 0, -1, 0);

	connect(UserBox::userboxmenu,SIGNAL(popup()),this,SLOT(PopupMenuCreate()));
}

Weather::~Weather()
{
	disconnect(UserBox::userboxmenu,SIGNAL(popup()),this,SLOT(PopupMenuCreate()));
	kadu->mainMenu()->removeItem(menuMyWeather);
	UserBox::userboxmenu->removeItem(UserBox::userboxmenu->getItem(tr("Show weather")));

	ConfigDialog::disconnectSlot("Weather", "Edit", SIGNAL(clicked()), this, SLOT(EditLocation()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(ApplyConfigDialog()));
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(CreateConfigDialog()));

	ConfigDialog::removeControl("Weather", "Edit");
	ConfigDialog::removeControl("Weather", "loc_buttons_box");
	ConfigDialog::removeControl("Weather", "loc_list");
	ConfigDialog::removeControl("Weather", "loc_box");
	
	ConfigDialog::removeTab("Weather");
	
	moduleConfig->sync();
	delete moduleConfig;
}

void Weather::CreateConfigDialog()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	lv_loc->setSorting(0);
	lv_loc->addColumn(tr("Contact"));
	lv_loc->addColumn(tr("Location code"));
	lv_loc->addColumn(tr("Weather server"));
	lv_loc->addColumn(tr("Config file"));
	lv_loc->setAllColumnsShowFocus(true);
	lv_loc->setColumnWidthMode(0, QListView::Maximum);
	lv_loc->setColumnWidthMode(1, QListView::Maximum);
	lv_loc->setColumnWidthMode(2, QListView::Maximum);
	lv_loc->setColumnWidthMode(3, QListView::Maximum);
	lv_loc->setResizeMode(QListView::LastColumn);
	//lv_loc->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Ignored));
	
	QPushButton* b_edit=ConfigDialog::getPushButton("Weather","Edit");
	b_edit->setDefault(true);
	
	QString configfile = moduleConfig->readEntry("Configs",config_file.readEntry("General","UIN"));
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+configfile));
	QString configname = wConfig.readEntry("Header","Name");
	
	new QListViewItem(lv_loc, config_file.readEntry("General","Nick"),
	moduleConfig->readEntry("Locations",config_file.readEntry("General","UIN")), configname, configfile);

	for(UserList::const_iterator u = userlist.begin(); u != userlist.end(); u++)
	{
		if((*u).uin() != 0)
		{
			QString configfile = moduleConfig->readEntry("Configs",QString().setNum((*u).uin()));
			ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+configfile));
			QString configname = wConfig.readEntry("Header","Name");
			new QListViewItem(lv_loc, (*u).altNick(), moduleConfig->readEntry("Locations",QString().setNum((*u).uin())),
				configname, configfile);
		}
	}
}

void Weather::ApplyConfigDialog()
{	
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* item = lv_loc->firstChild();
	
	QString myNick = config_file.readEntry("General","Nick");
	
	while(item)
	{
		QString altnick = item->text(0);
	
		if(altnick == myNick)
		{
			moduleConfig->writeEntry("Locations",config_file.readEntry("General","UIN"), item->text(1));
			moduleConfig->writeEntry("Configs",config_file.readEntry("General","UIN"), item->text(3));
		}
		else
		{
			const UserListElement& element = userlist.byAltNick(altnick);
			if(&element != NULL)
			{
				moduleConfig->writeEntry("Locations",QString().setNum(element.uin()), item->text(1));
				moduleConfig->writeEntry("Configs",QString().setNum(element.uin()), item->text(3));
			}
		}
		item = item->nextSibling();
	}
}

void Weather::EditLocation()
{
	QListView* lv_loc=ConfigDialog::getListView("Weather","loc_list");
	QListViewItem* item = lv_loc->currentItem();
	
	QString nick = item->text(0);
	int actual = weatherConfigFiles.findIndex(item->text(3));
	
	WeatherCodeDialog* wcd = new WeatherCodeDialog(nick, item, moduleConfig,
		weatherConfigFiles, weatherConfigNames, actual, lv_loc);
	wcd->show();
}

void Weather::PopupMenuCreate()
{
	UserBox *activeUserBox=kadu->userbox()->getActiveUserBox();
	if (activeUserBox==NULL)//to si zdarza...
		return;

	UserList users = activeUserBox->getSelectedUsers();	
	UserListElement user = (*users.begin());
	
	bool active = !moduleConfig->readEntry("Locations",QString().setNum(user.uin())).isNull();

	if(active){
		active = !moduleConfig->readEntry("Configs",QString().setNum(user.uin())).isNull();
		
		if(!active && !moduleConfig->readEntry("Weather","ServerConfigFile").isNull())
			active = true;
	}
	UserBox::userboxmenu->setItemEnabled(UserBox::userboxmenu->getItem(tr("Show weather")), active);
}

void Weather::ShowWeather()
{
	UserBox *activeUserBox=kadu->userbox()->getActiveUserBox();
	if (activeUserBox==NULL)//to si zdarza...
		return;
	
	UserList users = activeUserBox->getSelectedUsers();
	UserListElement user = (*users.begin());
	
	ShowForecastDialog* sfd = new ShowForecastDialog(QString().setNum(user.uin()), moduleConfig);
	sfd->show();
}

void Weather::ShowMyWeather()
{
	QString uin = config_file.readEntry("General","UIN");
	QString locid = moduleConfig->readEntry("Locations",uin);
	if(locid.isEmpty())
		return;
	
	ShowForecastDialog* sfd = new ShowForecastDialog(uin, moduleConfig);
	sfd->show();
}

